/***************************************************************************
 * scrape.c: Libxml2 based XPath evaluator.
 *   By Michael C McTernan, Michael.McTernan.2001@cs.bris.ac.uk
 *
 * This software is in the PUBLIC DOMAIN.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, without
 * any conditions or restrictions. This software is provided AS IS without
 * express or implied warranty.
 ***************************************************************************/

/**************************************************************************
 * Includes
 **************************************************************************/

#include <libxml/HTMLparser.h>
#include <libxml/xpath.h>
#include <stdlib.h>
#include <assert.h>
#include <unistd.h>

#if !defined(LIBXML_HTML_ENABLED)
#error libxml2 is missing LIBXML_HTML_ENABLED
#endif

/**************************************************************************
 * Local Functions
 **************************************************************************/

/** Print all the nodes in the passed set to some output stream.
 *
 */
static void printNodeSet(htmlDocPtr doc, xmlNodeSetPtr nodes, FILE *output)
{
    const int size = (nodes) ? nodes->nodeNr : 0;
    int       i;

    for(i = 0; i < size; i++)
    {
        assert(nodes->nodeTab[i]);
        htmlNodeDumpFile(output, doc, nodes->nodeTab[i]);
    }
}


/**
 *
 */
int runXPath(const xmlChar *xpathExpr, const char *file)
{
    htmlDocPtr         doc;
    xmlXPathContextPtr xpathCtx;
    xmlXPathObjectPtr  xpathObj;

    /* Quietly parse the document */
    doc = htmlReadFile(file,
                       NULL,
                       HTML_PARSE_RECOVER | /* This option is missing in older libxml2 */
                       HTML_PARSE_NOERROR |
                       HTML_PARSE_NOWARNING);
    if(doc == NULL)
    {
       fprintf(stderr, "Failed to parse document\n");
       return 2;
    }


    /* Create xpath evaluation context */
    xpathCtx = xmlXPathNewContext(doc);
    if(xpathCtx == NULL)
    {
        fprintf(stderr,"Error: unable to create new XPath context\n");
        xmlFreeDoc(doc);
        return 3;
    }

    /* Evaluate xpath expression */
    xpathObj = xmlXPathEvalExpression(xpathExpr, xpathCtx);
    if(xpathObj == NULL)
    {
        fprintf(stderr,"Error: unable to evaluate xpath expression \"%s\"\n", xpathExpr);
        xmlFreeDoc(doc);
        xmlXPathFreeContext(xpathCtx);
        return 4;
    }

    /* Print results */
    printNodeSet(doc, xpathObj->nodesetval, stdout);

    /* Free objects */
    xmlXPathFreeObject(xpathObj);
    xmlXPathFreeContext(xpathCtx);
    xmlFreeDoc(doc);

    return EXIT_SUCCESS;
}


int main(int argc, char *argv[])
{
    if(argc < 3)
    {
        printf("Usage: scrape <infile> <xpath>\n"
               "\n"
               "  <infile>  Input HTML file to parse.  If '-', input is read from stdin.\n"
               "  <xpath>   The XPath expression to evaluate against the document.\n"
               "\n"
               "Return Values:\n"
               "  0  Success.\n"
               "  1  Invalid input options.\n"
               "  2  Failed to parse input document.\n"
               "  3  Unable to setup parser.\n"
               "  4  Failed to parse XPath expression.\n"
               "\n"
               "Scrape by Michael C McTernan, Michael.McTernan.2001@cs.bris.ac.uk\n"
               "\n"
               "This software is in the PUBLIC DOMAIN. Permission to use, copy, modify, and\n"
               "distribute this software and its documentation for any purpose and without fee\n"
               "is hereby granted, without any conditions or restrictions. This software is\n"
               "provided AS IS without express or implied warranty.\n");

        return EXIT_FAILURE;
    }
    else
    {
        return runXPath(argv[2], argv[1]);
    }
}

